Argument(id::Base.UUID) = Argument(id, 0)

"""
    copy(t::AbstractDataTask)

Fallback implementation of the copy of an abstract data task, throwing an error.
"""
copy(t::AbstractDataTask) = error("Need to implement copying for your data tasks!")

"""
    copy(t::AbstractComputeTask)

Return a copy of the given compute task.
"""
copy(t::AbstractComputeTask) = typeof(t)(copy(t.func), copy(t.arguments))
copy(t::AbstractTaskFunction) = typeof(t)()

function ComputeTask(t::AbstractTaskFunction)
    return ComputeTask(t, Vector{Argument}())
end

"""
    copy(t::FusedComputeTask)

Return a copy of th egiven [`FusedComputeTask`](@ref).
"""
function copy(t::FusedComputeTask)
    return FusedComputeTask(copy(t.first_func), copy(t.second_func), copy(t.t1_inputs), t.t1_output, copy(t.t2_inputs))
end

function FusedComputeTask(T1::Type{<:AbstractComputeTask}, T2::Type{<:AbstractComputeTask}, t1_output::String)
    return FusedComputeTask(T1(), T2(), t1_output)
end
